
;*******************************************************************************
;                                                                              *
;    Filename: explight1572.asm                                                *
;    Date: 12/12/2017                                                          *
;    File Version:                                                             *
;    Author: Pete Griffiths                                                    *
;    Company:                                                                  *
;    Description: Switch fixed pulse width control for use with Exposure       *
;                 cycle lights                                                 *
;                                                                              *
;     When programming the 12LF1572 make sure Vdd is 3 volts - NOT 5 volts     *
;                                                                              *
;*******************************************************************************
;                                                                              *
;    Notes: In the MPLAB X Help, refer to the MPASM Assembler documentation    *
;    for information on assembly instructions.                                 *
;                                                                              *
;    Set TABS to 14 in IDE settings for nice formatting                        *
;                                                                              *
;*******************************************************************************
;                                                                              *
;    Known Issues: This template is designed for relocatable code.  As such,   *
;    build errors such as "Directive only allowed when generating an object    *
;    file" will result when the 'Build in Absolute Mode' checkbox is selected  *
;    in the project properties.                                                *
;                                                                              *
;*******************************************************************************
;                                                                              *
;    Revision History: 1                                                       *
;                                                                              *
;*******************************************************************************

	;list	p=12LF1572           ; list directive to define processor
	;

    errorlevel -302             ; disable bank warning messages
;
;*******************************************************************************

    #include "p12lf1572.inc"

; CONFIG1
; __config 0xF9E4
    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
; CONFIG2
; __config 0xF6FF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_ON & _BORV_LO & _LPBOREN_ON & _LVP_ON 

;*******************************************************************************
;
; MPLAB X has a feature which generates configuration bits source code.  Go to
; Window > PIC Memory Views > Configuration Bits.  Configure each field as
; needed and select 'Generate Source Code to Output'.  The resulting code which
; appears in the 'Output Window' > 'Config Bits Source' tab may be copied
; below.
;
;*******************************************************************************
;
; Variable Definitions
;
; Refer to datasheet for available data memory (RAM) organization assuming
; relocatible code organization (which is an option in project
; properties > mpasm (Global Options)).  Absolute mode generally should
; be used sparingly.
;
; Example of using GPR Uninitialized Data
;
;   GPR_VAR        UDATA
;   MYVAR1         RES        1      ; User variable linker places
;   MYVAR2         RES        1      ; User variable linker places
;   MYVAR3         RES        1      ; User variable linker places
;
;   ; Example of using Access Uninitialized Data Section (when available)
;   ; The variables for the context saving in the device datasheet may need
;   ; memory reserved here.
GPR_VAR        UDATA_OVR 0x20
; not defined any

SHARE_VAR      UDATA_SHR
var.iocaf_copy       RES 1	; working copy of IOCAF register
;

;---------------------------------------------------------------------------------------------------
;
; 	I/O Port Bit definitions
;
;---------------------------------------------------------------------------------------------------
bit.opto        equ           .2            ; I/O port bit driving Opto Isolator input 
bit.switchL     equ           .5            ; I/O port bit connected to switch input long pulse
bit.switchS     equ           .0            ; I/O port bit connected to switch input short pulse



;---------------------------------------------------------------------------------------------------
;
; 	Program CONSTANTs	
;
;---------------------------------------------------------------------------------------------------
; Opto isolator output active logic level
; Voltage drop on a PIC I/O pin when sinking is generally better than when sourcing current.
; Therefore it's better to use active low output, but choose what you want here.
; 1 -> active high ; PIC_IO--rlimit--optoLED--Vss
; 0 -> active low  ; Vdd--optoLED--rlimit--PIC_IO
OPTO_DRIVE		equ	0

 if OPTO_DRIVE
 	messg "OPTO_DRIVE is 1, active high"
 else
 	messg "OPTO_DRIVE is 0, active low"
 endif
 
; Configuration: Using 31Khz Fosc clock, TMR0 is clocked at Fosc/4.  TMR0 Prescaler is 1:32
; The timer counts up and overflows at 2^8 (256) so the number of 4.129mS ticks
; we count to get the desired delay is 256 - n_ticks

; Pulse_width_mS = (1 / 31) * 4 * 32 * (256-timer_value)
;                = 4.129mS x (256 - timer_value)

; Because the timer clock is 4.129mS the resolution for the delay is ~4mS. As an example you can't
; get a delay of 200mS, it falls in between 199-202mS so if you specify 200mS it will actually be 198mS 
; example: 4.129mS x (256 - 198) =  194mS
; example: 4.129mS x (256 - 199) =  198mS
; example: 4.129mS x (256 - 200) =  198mS
; example: 4.129mS x (256 - 201) =  198mS
; example: 4.129mS x (256 - 202) =  198mS
; example: 4.129mS x (256 - 203) =  202mS
;
; To calculate the timer load value for a desired delay in mS use:
; TMR_load = 256-((pulse_width_mS * 1000) / 4129)
;
; example: For 175mS the TMR0 load value is:
; 256 - ((175 * 1000) / 4129) = 173

; Exposure Strada light doesn't respond to a pulse width under approx. 140mS
; - A pulse over 405mS will set it to flashing mode
; - 395mS < pulse < 405mS unpredictable - may change mode, or may set to flash.
; - A pulse between approx 140mS and 395mS will switch power mode.
; - Two pulses over 140mS with a gap of less than 140mS can cause it to go into flashing mode.

; delay time in milliseconds ( . preceding the digits tells the assembler the radix is decimal)
OPTO_ON_TIME_SHORT    	equ         .229
OPTO_ON_TIME_LONG    	equ         .480
SWITCH_DEBOUNCE_TIME 	equ         .5
SWITCH_HOLDOFF_TIME  	equ         .146

;---------------------------------------------------------------------------------------------------
;
; 	MACRO definitions
;
;---------------------------------------------------------------------------------------------------
	
	; Delay calculation MACROs
	; Calculate TMR0 load value. Since the useful range of delays needed in this application 
	; is 4 to 500mS the calculated TMR0 load values will always be > 127.
	; Therefore we can pack bit 7 of the result to indicate if the output should be 
	; turned on (set to 1) or off (set to 0).  After output_control function has used the 
	; value of bit 7 to determine the required output state, it sets bit 7 to get the
	; correct TMR0 load value and passes it to the delay function.

M_bit_delayMs	macro 	bit7, delayMs
    ; use example: M_bit_delayMs 0, 200
	; Calculate TMR0 load value from delayMs, where delayMs is delay period in mS.
	; If bit7 is Zero, clear bit 7 of load vlaue to indicate opto isolator LED should be OFF, with delay
	; Else set bit 7 of load vlaue to indicate opto isolator LED should be ON, with delay
	if (delayMs >.520 || delayMs <.5)
	  error "delay_bit_delayMs MACRO: Delay time must between 5mS and 520mS"
	endif
	
	if (bit7 == 0)
	    movlw	.256-((delayMs *.1000)/.4129) & 0x7F
	else 
	    movlw	.256-((delayMs *.1000)/.4129) | 0x80
	endif
	call    	output_control
	endm


M_delay_only	macro 	td
	; Calculate TMR0 load value from td, where td is delay in mS.
	; Maximum delay has been increased to 1045mS since we call the timer_delay function
	; directly as bit 7 is not being packed with the opto LED setting. 
	if (td >.1045 || td <.5)
	  error "delay_only MACRO: Delay time must be between 5mS and 1045mS"
	endif
	movlw	.256-((td *.1000)/.4129) 
	call    	timer_delay
	endm

	; Macros to turn on/off the opto isolator LED.  Code is assembled with either output
	; active high, or active low depending on the value of the OPTO_DRIVE constant.
M_opto_on	macro
	; macro to turn on opto isolator output 
	if (OPTO_DRIVE)
	  bsf	LATA, bit.opto
	else
	  bcf	LATA, bit.opto
	endif
	endm
	
	
M_opto_off	macro
	; macro to turn off opto isolator output 
	if (OPTO_DRIVE)
	  bcf	LATA, bit.opto
	else
	  bsf	LATA, bit.opto
	endif
	endm
;---------------------------------------------------------------------------------------------------
;
; 	Reset Vector
;
;---------------------------------------------------------------------------------------------------

RES_VECT  	CODE    	0x0000       	; processor reset vector

	goto	resetInit     ; go to beginning of program
	
;---------------------------------------------------------------------------------------------------
;
; 	ISR - Interrupts are not used with this application
;
;---------------------------------------------------------------------------------------------------
;
ISR       	CODE    	0x0004	; interrupt vector location
           

	RETFIE
;---------------------------------------------------------------------------------------------------
;
; 	Reset initialisation code
;
;---------------------------------------------------------------------------------------------------

MAIN_PROG CODE ; let linker place main program

	; set internal clock to use 31KHz LFINTOSC - this is the uncalibrated LF oscillator
	; not the 31.25Khz clock dervied from the calibrated MFINTSOC
resetInit
	banksel	OSCCON
	movlw         (0<<IRCF3 | 0<<IRCF2 | 0<<IRCF1 |  0<<IRCF0  | 1<<SCS1 | 0<<SCS0); 31Khz Int Osc
	movwf         OSCCON
	
	;btfss   OSCSTAT, LFIOFR
	;bra	$-1		; loop until Low Frequency Internal Oscillator Ready bit is set

	; configure digital I/O on all ports
	banksel	ANSELA
	clrf	ANSELA
		
	; change bank register
	banksel       OPTION_REG
	; Weak Pull-ups enabled, prescaler to Timer0, PS=1:32,
	movlw         ( 0<<NOT_WPUEN | 0<<TMR0CS | 0<<PSA | 1<<PS2_OPTION_REG | 0<<PS1_OPTION_REG | 0<<PS0_OPTION_REG)
	movwf         OPTION_REG
	
	; Enable weak pull ups on required PortA inputs
	banksel	WPUA
	movlw	(1<<bit.switchL | 1<<bit.switchS)
	movwf	WPUA
		
	; set I/O output pin to turn off opto isolator
	banksel	LATA
	M_opto_off
	
	;bcf	LATA, bit.opto ; turn off opto isolator output
	
	banksel	TRISA
	movlw	(1<<bit.switchS | 1<<bit.switchL | 0<<bit.opto)
	movwf	TRISA
	
	; input threshold control. Set switch inputs to Schmitt Trigger CMOS thresholds 
	banksel INLVLA
	movlw   (1<<bit.switchS | 1<<bit.switchL)
	movwf   INLVLA

    ; enable slew rate control on opto output
    banksel SLRCONA
    bsf     SLRCONA, bit.opto

	; set interrupt-on-change for PORTA, bit.switch negative edge
	; PORTA has weak-pull up enabled so when switch is pressed input sees negative edge
	banksel	IOCAN
	movlw	1<<bit.switchS | 1<<bit.switchL
	movwf	IOCAN
	clrf	IOCAP
	
	; We set the Interrupt-on-change Interrupt Enable bit later to enable the wake-from-sleep.
	; We don't actually want interrupts enabled globally so GIE bit needs to be clear.
	; It will be cleared after a reset anyway but just to make the point that it shouldn't be
	; enabled I'm clearing it here.
	bcf	INTCON, GIE   ; Disable Global Interrupts
	

;---------------------------------------------------------------------------------------------------
;
; 	Main runtime code
;
;---------------------------------------------------------------------------------------------------

wait_switch 	; Go into sleep state and wait for falling edge on either
	; switch input to bring us out of sleep.
	banksel 	IOCAF
	clrf	IOCAF	; Clear interrupt-on-change flags
	banksel	INTCON
	bsf	INTCON, IOCIE ; Enable Interrupt on Change
	sleep		; Sleep to save power since we don't have a power
			; switch it will spend days/weeks in this state.
	nop
	; Make copy of the IOCAF register and work with that since it may
	; change if the 'other' switch is pressed during handling of this event
	banksel 	IOCAF
	movfw	IOCAF
	movwf	var.iocaf_copy
	
	; call debounce function twice 
	call	debounce
	skpnc
	bra	wait_switch
	
	call	debounce
	skpnc
	bra	wait_switch
	
	; Turn on opto isolator LED for specified time.
	; Identify which switch triggered event from IOCAF flags.
	; If both switches pressed switchL takes precedence
	btfsc	var.iocaf_copy, bit.switchL
	bra	pulse_long
	
pulse_short	M_bit_delayMs	1, OPTO_ON_TIME_SHORT
	bra	hold_off

pulse_long	M_bit_delayMs	1, OPTO_ON_TIME_LONG
	bra	hold_off

	; Turn off opto isolator LED and wait for holdoff time to prevent rapid toggling
	; the output.
	; ( in the case of the Exposure light, if the interval between two switch events is 
	; too short, it see's it as one long event and enters flash mode! )
hold_off	M_bit_delayMs	0, SWITCH_HOLDOFF_TIME

	; Test both switch inputs. If either switch is still pressed after hold_off period
	; wait for it to be released to prevent output toggling if switch is held down. 
	; We need to see both switches released regardless of which switch triggered the event.
	banksel	PORTA
release_check	btfss	PORTA, bit.switchL
	bra	release_check
	btfss	PORTA, bit.switchS
	bra	release_check
	
	bra 	wait_switch	; pulse event complete, go back and wait for next event 

;---------------------------------------------------------------------------------------------------
;
; 	Function definitions
;
;---------------------------------------------------------------------------------------------------

debounce	
	M_delay_only	SWITCH_DEBOUNCE_TIME
		
	clrc	; use carry flag to indictate if switch bounced
	
	; check the IOCAF flag to decide which switch input was pressed
	; If both switches pressed, switchL takes precedence
	
	btfsc	var.iocaf_copy, bit.switchL	
	bra	debounce_swL
	btfsc	var.iocaf_copy, bit.switchS	
	bra	debounce_swS
	
debounce_swS	
	banksel 	PORTA
	btfsc	PORTA, bit.switchS	; skip next if switch input is low
	setc	; else set carry if input is high (bouncing)
	return

debounce_swL	
	banksel 	PORTA
	btfsc	PORTA, bit.switchL	; skip next if switch input is low
	setc	; else set carry if input is high (bouncing)
	return
	
output_control
	; Output control and timer delay
	; Call with WREG bit 7 is Required opto-isolator LED state (1-on, 0-off)
	; WREG bit6-0 is TMR0 load value, bit 7 is set prior to entering timer_delay function
	; 
	banksel	LATA
	btfsc	WREG,7		
	M_opto_on		; bit 7 was set, so turn opto LED on
	
	btfss	WREG,7
	M_opto_off		; bit 7 was clear, so turn opto LED off
	
	iorlw	0x80		; set bit 7 always
	; continue through into timer_delay function

timer_delay	
    ; Load TMR0 with required delay value, clear TMR0IF then
	; wait for TMR0 to overflow, setting TMR0IF before returning.
	banksel	TMR0
	movwf	TMR0
	banksel	INTCON
	bcf	INTCON, TMR0IF
	
	btfss	INTCON, TMR0IF
	bra	$-1	; loop to previous instruction
	
	return  
	


    END